/* BresenClock.c */
/* Uhr als Bresenham-Animation. A. Christidis */
#define MORE_BRESEN

#include <conio.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "BresenClock.h"
#include "conDraw.h"
#include "GrafOps.h" 
#include "lineOps.h" 
#include "MatrOps.h" 

/*Globale Variablen: */
conClkHnd handHor, handMnt;
char      az=0, conBuffer[CON_YMAX+1][CON_XMAX+2]; /* [CON_XMAX+1 + '\0'] */
int       hour=2, mnt=10;
float     yCenter =(CON_YMAX + CON_YMIN)/2,
          xCenter =(CON_XMAX + CON_XMIN)/2;

/*************************************************************************/
 void conSetHands (int hour, int mnt)
/*************************************************************************/
{ /*Zeiger immer ausgehend von 12:00 h, Winkel ab der Vertikalen:*/
  float theta=0., dummy[4];

  /*Pos.Matrix Stundenzeiger:*/
  conHandInit (&handHor, LGTHHOR, SMBLHOR);
  /*Winkel je nach Uhrzeit:*/
#ifdef  MORE_BRESEN
  theta = (float)-(30.*hour); //12h*30/h=360
#endif  //MORE_BRESEN
  /*TranslationsMatrix an den Koord.-Ursprung:*/
  translate (-xCenter, -yCenter, 0, handHor.posMat);
  /*RotationsMatrix gegenueber 12:00-Stellung:*/
  rotate (theta, Z, handHor.posMat);
  /*Solange zentriert: Faktor 3/2 zur Korrektur der Schrift 8x12*/
  scale (3./2., 1., 1., handHor.posMat);
  /*TranslationsMatrix zurueck ans Zentrum:*/
  translate (xCenter, yCenter, 0, handHor.posMat);

  /*Koord. Stundenzeiger:*/
  /* 12:00-Positionierung von links multiplizieren: */
  matMul (1., handHor.posMat, 4, 4, 1., handHor.vtx[1], 1, dummy);
  saveMatAs (dummy, handHor.vtx[1], 4);

  /*Minutenzeiger:*/
  conHandInit (&handMnt, LGTHMNT, SMBLMNT); 
  theta = (float)-6.*mnt;
  /*TranslationsMatrix an den Koord.-Ursprung:*/
  translate (-xCenter, -yCenter, 0, handMnt.posMat);
  /*RotationsMatrix gegenueber 12:00-Stellung:*/
  rotate (theta, Z, handMnt.posMat);
  /*Solange zentriert: Faktor 3/2 zur Korrektur der Schrift 8x12*/
  scale (3./2., 1., 1., handMnt.posMat);
  /*TranslationsMatrix zurueck ans Zentrum:*/
  translate (xCenter, yCenter, 0, handMnt.posMat);

  /*Koord. Minutenzeiger:*/
  /* 12:00-Positionierung von links multiplizieren: */
  matMul (1., handMnt.posMat, 4, 4, 1., handMnt.vtx[1], 1, dummy);
  saveMatAs (dummy, handMnt.vtx[1], 4);
  /*Faktor 3/2 zur Korrektur der Schrift 8x12*/

  return;
}

/*************************************************************************/
 void conHandInit (conClkHnd *hand, float length, char symbol)
/*************************************************************************/
{ /*Zeiger auf 12:00:*/
  int jV=0, jD=0; // counter Vertices, Dimensions

  for (jV=0; jV<2; jV++)
  { for (jD=0; jD<4; jD++)
    { hand->vtx[jV][jD] = 0.;
    }
  }
  hand->vtx[0][X] = xCenter;
  hand->vtx[0][Y] = yCenter;
  hand->vtx[0][3] = 1.; // homogene Koordinate
  hand->vtx[1][X] = xCenter;
  hand->vtx[1][Y] = yCenter+length;
  hand->vtx[1][3] = 1.; // homogene Koordinate
  hand->symbol = symbol; // '\333'; //'\262';

  Identity (hand->posMat, 4);
  return;
}

/*************************************************************************/
  void conDrawHands (void)
/*************************************************************************/
{ conSetPixel ((int)handMnt.vtx[1][X], (int)handMnt.vtx[1][Y], handMnt.symbol);
  conSetPixel ((int)handHor.vtx[1][X], (int)handHor.vtx[1][Y], handHor.symbol);
  MidpointLine ((int)xCenter, (int)yCenter, (int)(handMnt.vtx[1][X]), 
                                    (int)(handMnt.vtx[1][Y]), handMnt.symbol);
  MidpointLine ((int)xCenter, (int)yCenter, (int)(handHor.vtx[1][X]), 
                                    (int)(handHor.vtx[1][Y]), handHor.symbol);
  conSetPixel ((int)(xCenter), (int)(yCenter), handHor.symbol);
  return;
}

/*************************************************************************/
   void conInit(void)
/*************************************************************************/
{ system (CON_SIZE);
  conHandInit (&handMnt, LGTHMNT, SMBLMNT); 
  conHandInit (&handHor, LGTHHOR, SMBLHOR); 
  conClrBuff(); 
  conClrScr(); 
  return;
}

/*************************************************************************/
  void draw (void)
/*************************************************************************/
{ conDrawClock();
  conSetHands (hour, mnt); 
  conDrawHands ();
  conSwapBuff ();
  conClrBuff();
}

/*************************************************************************/
  int main()
/*************************************************************************/
{ int keypress=0;
  conInit();
 printf ("\n  Aris Christidis:\n\n\r  console BresenClock\
           \n\r  [run: <CR>, quit: <ESC>]\
           \n\n\r  (press any key)"); _getch();
  while (keypress != ESC)
  { draw();
    keypress = _getch();
    key ((char)keypress);
  }
  return 0;
}

/*=======================================================================*/
/*Weniger interessante Funktionen:*/

/*************************************************************************/
  void key (char k)
/*************************************************************************/
{ switch (k)
  { case ' ': az   = 1-az;    /* A&Z: Buffer Start&End */
              break;
    case CR:  mnt += 5; 
              if (mnt >= 60) { mnt -= 60;  hour++; if (hour > 12) hour-=12; }
              break;
    case ESC: exit(0);        /* Escape */
  }
}

/*************************************************************************/
  void conDrawClock (void)
/*************************************************************************/
{ conBuffer[(int)(yCenter)][(int)(xCenter)]   = 'O';

  /*Faktor 3/2 zur Korrektur der Schrift 8x12*/
  conBuffer[(int)(yCenter+CLK_RAD)][(int)(xCenter)]   = '1';
  conBuffer[(int)(yCenter+CLK_RAD)][(int)(xCenter+1)] = '2';

  conBuffer[(int)(yCenter+CLK_RAD*SIN(60))][(int)(xCenter+CLK_RAD*COS(60)*3/2)+1] = '1';
  conBuffer[(int)(yCenter+CLK_RAD*SIN(60))][(int)(xCenter-CLK_RAD*COS(60)*3/2)-1] = '1';
  conBuffer[(int)(yCenter+CLK_RAD*SIN(60))][(int)(xCenter-CLK_RAD*COS(60)*3/2)]   = '1';

  conBuffer[(int)(yCenter+CLK_RAD*SIN(30))][(int)(xCenter+CLK_RAD*COS(30)*3/2)+1] = '2';
  conBuffer[(int)(yCenter+CLK_RAD*SIN(30))][(int)(xCenter-CLK_RAD*COS(30)*3/2)-1] = '1';
  conBuffer[(int)(yCenter+CLK_RAD*SIN(30))][(int)(xCenter-CLK_RAD*COS(30)*3/2)]   = '0';

  conBuffer[(int)(yCenter)][(int)(xCenter+CLK_RAD*3/2)+1] = '3';
  conBuffer[(int)(yCenter)][(int)(xCenter-CLK_RAD*3/2)-1] = '9';

  conBuffer[(int)(yCenter-CLK_RAD*SIN(30))][(int)(xCenter+CLK_RAD*COS(30)*3/2)+1] = '4';
  conBuffer[(int)(yCenter-CLK_RAD*SIN(30))][(int)(xCenter-CLK_RAD*COS(30)*3/2)]   = '8';

  conBuffer[(int)(yCenter-CLK_RAD*SIN(60))][(int)(xCenter+CLK_RAD*COS(60)*3/2)+1] = '5';
  conBuffer[(int)(yCenter-CLK_RAD*SIN(60))][(int)(xCenter-CLK_RAD*COS(60)*3/2)]   = '7';

  conBuffer[(int)(yCenter-CLK_RAD)][(int)(xCenter)] = '6';

  return;
}
